# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.bad_request import BadRequest
from ..types.conflict import Conflict
from ..types.forbidden import Forbidden
from ..types.include_total_count import IncludeTotalCount
from ..types.not_found import NotFound
from ..types.request_after_opaque_cursor import RequestAfterOpaqueCursor
from ..types.request_page_size import RequestPageSize
from ..types.too_many_requests import TooManyRequests
from ..types.unauthorized import Unauthorized
from ..types.uuid_ import Uuid
from ..types.version_stack_assets_with_includes_response import VersionStackAssetsWithIncludesResponse
from ..types.version_stack_copy_response import VersionStackCopyResponse
from ..types.version_stack_response import VersionStackResponse
from ..types.version_stack_with_includes_response import VersionStackWithIncludesResponse
from ..types.version_stacks_with_includes_response import VersionStacksWithIncludesResponse
from .types.version_stack_copy_params_data import VersionStackCopyParamsData
from .types.version_stack_create_params_data import VersionStackCreateParamsData
from .types.version_stack_move_params_data import VersionStackMoveParamsData
from .types.version_stacks_index_request_include import VersionStacksIndexRequestInclude
from .types.version_stacks_list_request_include import VersionStacksListRequestInclude
from .types.version_stacks_show_request_include import VersionStacksShowRequestInclude

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVersionStacksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def index(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        include: typing.Optional[VersionStacksIndexRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStackAssetsWithIncludesResponse]:
        """
        List the children (files) in a given version stack.
        <br>
        Use the `include` query parameter to selectively include additional properties in the response.
        <br>
        Note: if you include `media_links.original` and the user does not have permission to download files
        in the corresponding project, then this endpoint will respond with a `403 Forbidden` error.
        If the content is inaccessible because watermarking is required for this user and isn't supported by
        the requested media_links, then the request will succeed but the unsupported media links will be set to null.
        Similarly, if a requested transcode link does not exist for a particular file (e.g. including
        `media_links.video_h264_180` on a static image file) or transoding process hasn't finished
        (i.e. the file's `status` is "uploaded" rather than "transcoded"), then the a media link will also be set to
        null in the response payload. In short, the client must handle null media links gracefully.
        <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        include : typing.Optional[VersionStacksIndexRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStackAssetsWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/children",
            method="GET",
            params={
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackAssetsWithIncludesResponse,
                    construct_type(
                        type_=VersionStackAssetsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def show(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        include: typing.Optional[VersionStacksShowRequestInclude] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStackWithIncludesResponse]:
        """
        Show version stack details. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        include : typing.Optional[VersionStacksShowRequestInclude]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStackWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}",
            method="GET",
            params={
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackWithIncludesResponse,
                    construct_type(
                        type_=VersionStackWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def copy(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        copy_metadata: typing.Optional[bool] = None,
        data: typing.Optional[VersionStackCopyParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStackCopyResponse]:
        """
        Copy version stack. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        copy_metadata : typing.Optional[bool]
            Whether to copy metadata values along with the version stack

        data : typing.Optional[VersionStackCopyParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStackCopyResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/copy",
            method="POST",
            params={
                "copy_metadata": copy_metadata,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackCopyParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackCopyResponse,
                    construct_type(
                        type_=VersionStackCopyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self,
        account_id: Uuid,
        folder_id: Uuid,
        *,
        include: typing.Optional[VersionStacksListRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStacksWithIncludesResponse]:
        """
        List version stacks in a given folder.  <br>Rate Limits: 5 calls per 1 second(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        folder_id : Uuid


        include : typing.Optional[VersionStacksListRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStacksWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/folders/{jsonable_encoder(folder_id)}/version_stacks",
            method="GET",
            params={
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStacksWithIncludesResponse,
                    construct_type(
                        type_=VersionStacksWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        account_id: Uuid,
        folder_id: Uuid,
        *,
        data: VersionStackCreateParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStackWithIncludesResponse]:
        """
        Create a new Version Stack under the parent folder. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        folder_id : Uuid


        data : VersionStackCreateParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStackWithIncludesResponse]
            Created
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/folders/{jsonable_encoder(folder_id)}/version_stacks",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackCreateParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackWithIncludesResponse,
                    construct_type(
                        type_=VersionStackWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def move(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        data: VersionStackMoveParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionStackResponse]:
        """
        Move version stack to a folder. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        data : VersionStackMoveParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionStackResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/move",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackMoveParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackResponse,
                    construct_type(
                        type_=VersionStackResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVersionStacksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def index(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        include: typing.Optional[VersionStacksIndexRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStackAssetsWithIncludesResponse]:
        """
        List the children (files) in a given version stack.
        <br>
        Use the `include` query parameter to selectively include additional properties in the response.
        <br>
        Note: if you include `media_links.original` and the user does not have permission to download files
        in the corresponding project, then this endpoint will respond with a `403 Forbidden` error.
        If the content is inaccessible because watermarking is required for this user and isn't supported by
        the requested media_links, then the request will succeed but the unsupported media links will be set to null.
        Similarly, if a requested transcode link does not exist for a particular file (e.g. including
        `media_links.video_h264_180` on a static image file) or transoding process hasn't finished
        (i.e. the file's `status` is "uploaded" rather than "transcoded"), then the a media link will also be set to
        null in the response payload. In short, the client must handle null media links gracefully.
        <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        include : typing.Optional[VersionStacksIndexRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStackAssetsWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/children",
            method="GET",
            params={
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackAssetsWithIncludesResponse,
                    construct_type(
                        type_=VersionStackAssetsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def show(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        include: typing.Optional[VersionStacksShowRequestInclude] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStackWithIncludesResponse]:
        """
        Show version stack details. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        include : typing.Optional[VersionStacksShowRequestInclude]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStackWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}",
            method="GET",
            params={
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackWithIncludesResponse,
                    construct_type(
                        type_=VersionStackWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def copy(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        copy_metadata: typing.Optional[bool] = None,
        data: typing.Optional[VersionStackCopyParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStackCopyResponse]:
        """
        Copy version stack. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        copy_metadata : typing.Optional[bool]
            Whether to copy metadata values along with the version stack

        data : typing.Optional[VersionStackCopyParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStackCopyResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/copy",
            method="POST",
            params={
                "copy_metadata": copy_metadata,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackCopyParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackCopyResponse,
                    construct_type(
                        type_=VersionStackCopyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self,
        account_id: Uuid,
        folder_id: Uuid,
        *,
        include: typing.Optional[VersionStacksListRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStacksWithIncludesResponse]:
        """
        List version stacks in a given folder.  <br>Rate Limits: 5 calls per 1 second(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        folder_id : Uuid


        include : typing.Optional[VersionStacksListRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStacksWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/folders/{jsonable_encoder(folder_id)}/version_stacks",
            method="GET",
            params={
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStacksWithIncludesResponse,
                    construct_type(
                        type_=VersionStacksWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        account_id: Uuid,
        folder_id: Uuid,
        *,
        data: VersionStackCreateParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStackWithIncludesResponse]:
        """
        Create a new Version Stack under the parent folder. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        folder_id : Uuid


        data : VersionStackCreateParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStackWithIncludesResponse]
            Created
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/folders/{jsonable_encoder(folder_id)}/version_stacks",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackCreateParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackWithIncludesResponse,
                    construct_type(
                        type_=VersionStackWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def move(
        self,
        account_id: Uuid,
        version_stack_id: Uuid,
        *,
        data: VersionStackMoveParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionStackResponse]:
        """
        Move version stack to a folder. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        version_stack_id : Uuid


        data : VersionStackMoveParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionStackResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/version_stacks/{jsonable_encoder(version_stack_id)}/move",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=VersionStackMoveParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionStackResponse,
                    construct_type(
                        type_=VersionStackResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .share import Share


class AddAssetResponseData(Share):
    asset_added: str = pydantic.Field()
    """
    ID for asset added
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .time_stamp import TimeStamp


class Comment(UncheckedBaseModel):
    annotation: typing.Optional[str] = None
    completed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Completion timestamp
    """

    completer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of user who marked the comment as completed
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    file_id: str = pydantic.Field()
    """
    File ID
    """

    id: str = pydantic.Field()
    """
    Comment ID
    """

    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    Document page
    """

    text: str = pydantic.Field()
    """
    Comment text
    """

    text_edited_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Text edited timestamp
    """

    timestamp: TimeStamp
    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.unchecked_base_model import UncheckedBaseModel


class BulkUpdateMetadataParamsDataValuesItem(UncheckedBaseModel):
    field_definition_id: str
    value: typing.Any

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.bad_request import BadRequest
from ..types.comment_response import CommentResponse
from ..types.comment_with_includes import CommentWithIncludes
from ..types.comment_with_includes_response import CommentWithIncludesResponse
from ..types.comments_with_includes_response import CommentsWithIncludesResponse
from ..types.conflict import Conflict
from ..types.forbidden import Forbidden
from ..types.include_total_count import IncludeTotalCount
from ..types.not_found import NotFound
from ..types.request_after_opaque_cursor import RequestAfterOpaqueCursor
from ..types.request_page_size import RequestPageSize
from ..types.too_many_requests import TooManyRequests
from ..types.unauthorized import Unauthorized
from ..types.uuid_ import Uuid
from .types.comments_index_request_include import CommentsIndexRequestInclude
from .types.comments_show_request_include import CommentsShowRequestInclude
from .types.create_comment_params_data import CreateCommentParamsData
from .types.update_comment_params_data import UpdateCommentParamsData

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCommentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def show(
        self,
        account_id: Uuid,
        comment_id: Uuid,
        *,
        timestamp_as_timecode: typing.Optional[bool] = None,
        include: typing.Optional[CommentsShowRequestInclude] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CommentWithIncludesResponse]:
        """
        Show a single comment on a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        timestamp_as_timecode : typing.Optional[bool]


        include : typing.Optional[CommentsShowRequestInclude]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CommentWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="GET",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentWithIncludesResponse,
                    construct_type(
                        type_=CommentWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, account_id: Uuid, comment_id: Uuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete comment from an asset. <br>Rate Limits: 60 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        account_id: Uuid,
        comment_id: Uuid,
        *,
        data: UpdateCommentParamsData,
        timestamp_as_timecode: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CommentResponse]:
        """
        Update comment on given asset. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        data : UpdateCommentParamsData

        timestamp_as_timecode : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CommentResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="PATCH",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=UpdateCommentParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentResponse,
                    construct_type(
                        type_=CommentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def index(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        timestamp_as_timecode: typing.Optional[bool] = None,
        include: typing.Optional[CommentsIndexRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CommentWithIncludes, CommentsWithIncludesResponse]:
        """
        List comments on a given asset. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        timestamp_as_timecode : typing.Optional[bool]


        include : typing.Optional[CommentsIndexRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CommentWithIncludes, CommentsWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/files/{jsonable_encoder(file_id)}/comments",
            method="GET",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    CommentsWithIncludesResponse,
                    construct_type(
                        type_=CommentsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.links is not None:
                    _parsed_next = _parsed_response.links.next
                    _has_next = _parsed_next is not None and _parsed_next != ""

                    def _fetch_next_page(_next_path):
                        _next_response = self._client_wrapper.httpx_client.request(method="GET", path=_next_path)
                        _next_parsed = typing.cast(
                            CommentsWithIncludesResponse,
                            construct_type(
                                type_=CommentsWithIncludesResponse,  # type: ignore
                                object_=_next_response.json(),
                            ),
                        )
                        _next_items = _next_parsed.data
                        _next_next = _next_parsed.links.next if _next_parsed.links is not None else None
                        _next_has_next = _next_next is not None and _next_next != ""
                        return SyncPager(
                            has_next=_next_has_next,
                            items=_next_items,
                            get_next=(lambda _p=_next_next: _fetch_next_page(_p)) if _next_has_next else None,
                            response=_next_parsed,
                        )

                    _get_next = lambda _p=_parsed_next: _fetch_next_page(_p)
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        data: CreateCommentParamsData,
        timestamp_as_timecode: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CommentResponse]:
        """
        Create a comment on a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        data : CreateCommentParamsData

        timestamp_as_timecode : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CommentResponse]
            Created
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/files/{jsonable_encoder(file_id)}/comments",
            method="POST",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=CreateCommentParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentResponse,
                    construct_type(
                        type_=CommentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCommentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def show(
        self,
        account_id: Uuid,
        comment_id: Uuid,
        *,
        timestamp_as_timecode: typing.Optional[bool] = None,
        include: typing.Optional[CommentsShowRequestInclude] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CommentWithIncludesResponse]:
        """
        Show a single comment on a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        timestamp_as_timecode : typing.Optional[bool]


        include : typing.Optional[CommentsShowRequestInclude]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CommentWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="GET",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentWithIncludesResponse,
                    construct_type(
                        type_=CommentWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, account_id: Uuid, comment_id: Uuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete comment from an asset. <br>Rate Limits: 60 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        account_id: Uuid,
        comment_id: Uuid,
        *,
        data: UpdateCommentParamsData,
        timestamp_as_timecode: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CommentResponse]:
        """
        Update comment on given asset. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        comment_id : Uuid


        data : UpdateCommentParamsData

        timestamp_as_timecode : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CommentResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/comments/{jsonable_encoder(comment_id)}",
            method="PATCH",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=UpdateCommentParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentResponse,
                    construct_type(
                        type_=CommentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def index(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        timestamp_as_timecode: typing.Optional[bool] = None,
        include: typing.Optional[CommentsIndexRequestInclude] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CommentWithIncludes, CommentsWithIncludesResponse]:
        """
        List comments on a given asset. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        timestamp_as_timecode : typing.Optional[bool]


        include : typing.Optional[CommentsIndexRequestInclude]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CommentWithIncludes, CommentsWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/files/{jsonable_encoder(file_id)}/comments",
            method="GET",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
                "include": include,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    CommentsWithIncludesResponse,
                    construct_type(
                        type_=CommentsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.links is not None:
                    _parsed_next = _parsed_response.links.next
                    _has_next = _parsed_next is not None and _parsed_next != ""

                    async def _fetch_next_page(_next_path):
                        _next_response = await self._client_wrapper.httpx_client.request(method="GET", path=_next_path)
                        _next_parsed = typing.cast(
                            CommentsWithIncludesResponse,
                            construct_type(
                                type_=CommentsWithIncludesResponse,  # type: ignore
                                object_=_next_response.json(),
                            ),
                        )
                        _next_items = _next_parsed.data
                        _next_next = _next_parsed.links.next if _next_parsed.links is not None else None
                        _next_has_next = _next_next is not None and _next_next != ""
                        return AsyncPager(
                            has_next=_next_has_next,
                            items=_next_items,
                            get_next=(lambda _p=_next_next: _fetch_next_page(_p)) if _next_has_next else None,
                            response=_next_parsed,
                        )

                    _get_next = lambda _p=_parsed_next: _fetch_next_page(_p)
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        data: CreateCommentParamsData,
        timestamp_as_timecode: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CommentResponse]:
        """
        Create a comment on a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        data : CreateCommentParamsData

        timestamp_as_timecode : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CommentResponse]
            Created
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/files/{jsonable_encoder(file_id)}/comments",
            method="POST",
            params={
                "timestamp_as_timecode": timestamp_as_timecode,
            },
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=CreateCommentParamsData, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CommentResponse,
                    construct_type(
                        type_=CommentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        construct_type(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        construct_type(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        construct_type(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        construct_type(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        construct_type(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        construct_type(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from ...types.date_definition_params_field_configuration import DateDefinitionParamsFieldConfiguration
from ...types.number_definition_params_field_configuration import NumberDefinitionParamsFieldConfiguration
from ...types.rating_definition_params_field_configuration import RatingDefinitionParamsFieldConfiguration
from ...types.select_definition_params_field_configuration import SelectDefinitionParamsFieldConfiguration
from ...types.select_multi_definition_params_field_configuration import SelectMultiDefinitionParamsFieldConfiguration
from ...types.toggle_definition_params_field_configuration import ToggleDefinitionParamsFieldConfiguration
from ...types.user_multi_definition_params_field_configuration import UserMultiDefinitionParamsFieldConfiguration
from ...types.user_single_definition_params_field_configuration import UserSingleDefinitionParamsFieldConfiguration


class CreateFieldDefinitionParamsData_Date(UncheckedBaseModel):
    field_type: typing.Literal["date"] = "date"
    field_configuration: DateDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_LongText(UncheckedBaseModel):
    field_type: typing.Literal["long_text"] = "long_text"
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_Number(UncheckedBaseModel):
    field_type: typing.Literal["number"] = "number"
    field_configuration: NumberDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_Rating(UncheckedBaseModel):
    field_type: typing.Literal["rating"] = "rating"
    field_configuration: RatingDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_Select(UncheckedBaseModel):
    field_type: typing.Literal["select"] = "select"
    field_configuration: SelectDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_SelectMulti(UncheckedBaseModel):
    field_type: typing.Literal["select_multi"] = "select_multi"
    field_configuration: SelectMultiDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_Text(UncheckedBaseModel):
    field_type: typing.Literal["text"] = "text"
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_Toggle(UncheckedBaseModel):
    field_type: typing.Literal["toggle"] = "toggle"
    field_configuration: typing.Optional[ToggleDefinitionParamsFieldConfiguration] = None
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_UserMulti(UncheckedBaseModel):
    field_type: typing.Literal["user_multi"] = "user_multi"
    field_configuration: UserMultiDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class CreateFieldDefinitionParamsData_UserSingle(UncheckedBaseModel):
    field_type: typing.Literal["user_single"] = "user_single"
    field_configuration: UserSingleDefinitionParamsFieldConfiguration
    name: str

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


CreateFieldDefinitionParamsData = typing_extensions.Annotated[
    typing.Union[
        CreateFieldDefinitionParamsData_Date,
        CreateFieldDefinitionParamsData_LongText,
        CreateFieldDefinitionParamsData_Number,
        CreateFieldDefinitionParamsData_Rating,
        CreateFieldDefinitionParamsData_Select,
        CreateFieldDefinitionParamsData_SelectMulti,
        CreateFieldDefinitionParamsData_Text,
        CreateFieldDefinitionParamsData_Toggle,
        CreateFieldDefinitionParamsData_UserMulti,
        CreateFieldDefinitionParamsData_UserSingle,
    ],
    UnionMetadata(discriminant="field_type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.metadata_response import MetadataResponse
from ..types.uuid_ import Uuid
from .raw_client import AsyncRawMetadataClient, RawMetadataClient
from .types.bulk_update_metadata_params_data import BulkUpdateMetadataParamsData

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MetadataClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMetadataClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMetadataClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMetadataClient
        """
        return self._raw_client

    def bulk_update(
        self,
        account_id: Uuid,
        project_id: Uuid,
        *,
        data: BulkUpdateMetadataParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update metadata values across multiple files. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        project_id : Uuid


        data : BulkUpdateMetadataParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from frameio import Frameio
        from frameio.metadata import (
            BulkUpdateMetadataParamsData,
            BulkUpdateMetadataParamsDataValuesItem,
        )

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata.bulk_update(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            project_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            data=BulkUpdateMetadataParamsData(
                file_ids=[
                    "6aa84ef8-7f0a-431e-805e-a5e96345a176",
                    "54916a83-698a-43f8-9f14-19fd99094474",
                ],
                values=[
                    BulkUpdateMetadataParamsDataValuesItem(
                        field_definition_id="79e05ece-741a-4a20-b320-3a09c15673bb",
                        value=[
                            {
                                "id": "95681686-3627-4b8c-9a7c-982e41dc8af6",
                                "type": "user",
                            },
                            {
                                "id": "8759b175-9d71-41af-8b52-0dc79db384ec",
                                "type": "account_user_group",
                            },
                        ],
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.bulk_update(account_id, project_id, data=data, request_options=request_options)
        return _response.data

    def show(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        show_null: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MetadataResponse:
        """
        Show the metadata of a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        show_null : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetadataResponse
            OK

        Examples
        --------
        from frameio import Frameio

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata.show(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            file_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            show_null=True,
        )
        """
        _response = self._raw_client.show(account_id, file_id, show_null=show_null, request_options=request_options)
        return _response.data


class AsyncMetadataClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMetadataClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMetadataClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMetadataClient
        """
        return self._raw_client

    async def bulk_update(
        self,
        account_id: Uuid,
        project_id: Uuid,
        *,
        data: BulkUpdateMetadataParamsData,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update metadata values across multiple files. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        project_id : Uuid


        data : BulkUpdateMetadataParamsData

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from frameio import AsyncFrameio
        from frameio.metadata import (
            BulkUpdateMetadataParamsData,
            BulkUpdateMetadataParamsDataValuesItem,
        )

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata.bulk_update(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                project_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                data=BulkUpdateMetadataParamsData(
                    file_ids=[
                        "6aa84ef8-7f0a-431e-805e-a5e96345a176",
                        "54916a83-698a-43f8-9f14-19fd99094474",
                    ],
                    values=[
                        BulkUpdateMetadataParamsDataValuesItem(
                            field_definition_id="79e05ece-741a-4a20-b320-3a09c15673bb",
                            value=[
                                {
                                    "id": "95681686-3627-4b8c-9a7c-982e41dc8af6",
                                    "type": "user",
                                },
                                {
                                    "id": "8759b175-9d71-41af-8b52-0dc79db384ec",
                                    "type": "account_user_group",
                                },
                            ],
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_update(
            account_id, project_id, data=data, request_options=request_options
        )
        return _response.data

    async def show(
        self,
        account_id: Uuid,
        file_id: Uuid,
        *,
        show_null: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MetadataResponse:
        """
        Show the metadata of a file. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        file_id : Uuid


        show_null : typing.Optional[bool]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetadataResponse
            OK

        Examples
        --------
        import asyncio

        from frameio import AsyncFrameio

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata.show(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                file_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                show_null=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.show(
            account_id, file_id, show_null=show_null, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .audit_logwith_includes import AuditLogwithIncludes
from .links import Links


class AuditLogsWithIncludesResponse(UncheckedBaseModel):
    """
    Frame.io Audit log events
    """

    data: typing.List[AuditLogwithIncludes] = pydantic.Field()
    """
    Audit logs
    """

    links: Links
    total_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total count
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel


class MediaLinkCommon(UncheckedBaseModel):
    download_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL to download the media file.<br>
    HTTP response headers will include Content-Disposition = 'attachment;filename=<filename>'.<br>
    Watermarks may be applied for transcode links as per Account settings and User permissions.<br>
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .workspace import Workspace


class WorkspaceResponse(UncheckedBaseModel):
    """
    Frame.io workspace details
    """

    data: Workspace

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .webhook_create_response_data import WebhookCreateResponseData


class WebhookCreateResponse(UncheckedBaseModel):
    """
    Returns a single webhook with secret.
    """

    data: WebhookCreateResponseData

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .number_definition_field_configuration_number_format import NumberDefinitionFieldConfigurationNumberFormat


class NumberDefinitionFieldConfiguration(UncheckedBaseModel):
    number_format: typing.Optional[NumberDefinitionFieldConfigurationNumberFormat] = pydantic.Field(default=None)
    """
    Number format
    """

    scale: int = pydantic.Field()
    """
    Number scale. Indicates the number of decimal places of precision for the number value
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .date_definition_field_configuration import DateDefinitionFieldConfiguration
from .long_text_definition_field_configuration import LongTextDefinitionFieldConfiguration
from .number_definition_field_configuration import NumberDefinitionFieldConfiguration
from .rating_definition_field_configuration import RatingDefinitionFieldConfiguration
from .select_definition_field_configuration import SelectDefinitionFieldConfiguration
from .select_multi_definition_field_configuration import SelectMultiDefinitionFieldConfiguration
from .text_definition_field_configuration import TextDefinitionFieldConfiguration
from .toggle_definition_field_configuration import ToggleDefinitionFieldConfiguration
from .user_multi_definition_field_configuration import UserMultiDefinitionFieldConfiguration
from .user_single_definition_field_configuration import UserSingleDefinitionFieldConfiguration


class FieldDefinition_Date(UncheckedBaseModel):
    field_type: typing.Literal["date"] = "date"
    created_at: dt.datetime
    creator_id: str
    field_configuration: DateDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_LongText(UncheckedBaseModel):
    field_type: typing.Literal["long_text"] = "long_text"
    created_at: dt.datetime
    creator_id: str
    field_configuration: LongTextDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_Number(UncheckedBaseModel):
    field_type: typing.Literal["number"] = "number"
    created_at: dt.datetime
    creator_id: str
    field_configuration: NumberDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_Rating(UncheckedBaseModel):
    field_type: typing.Literal["rating"] = "rating"
    created_at: dt.datetime
    creator_id: str
    field_configuration: RatingDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_Select(UncheckedBaseModel):
    field_type: typing.Literal["select"] = "select"
    created_at: dt.datetime
    creator_id: str
    field_configuration: SelectDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_SelectMulti(UncheckedBaseModel):
    field_type: typing.Literal["select_multi"] = "select_multi"
    created_at: dt.datetime
    creator_id: str
    field_configuration: SelectMultiDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_Text(UncheckedBaseModel):
    field_type: typing.Literal["text"] = "text"
    created_at: dt.datetime
    creator_id: str
    field_configuration: TextDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_Toggle(UncheckedBaseModel):
    field_type: typing.Literal["toggle"] = "toggle"
    created_at: dt.datetime
    creator_id: str
    field_configuration: ToggleDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_UserMulti(UncheckedBaseModel):
    field_type: typing.Literal["user_multi"] = "user_multi"
    created_at: dt.datetime
    creator_id: str
    field_configuration: UserMultiDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


class FieldDefinition_UserSingle(UncheckedBaseModel):
    field_type: typing.Literal["user_single"] = "user_single"
    created_at: dt.datetime
    creator_id: str
    field_configuration: UserSingleDefinitionFieldConfiguration
    id: str
    mutable: bool
    name: str
    updated_at: dt.datetime

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)


FieldDefinition = typing_extensions.Annotated[
    typing.Union[
        FieldDefinition_Date,
        FieldDefinition_LongText,
        FieldDefinition_Number,
        FieldDefinition_Rating,
        FieldDefinition_Select,
        FieldDefinition_SelectMulti,
        FieldDefinition_Text,
        FieldDefinition_Toggle,
        FieldDefinition_UserMulti,
        FieldDefinition_UserSingle,
    ],
    UnionMetadata(discriminant="field_type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .rating_definition_field_configuration_style import RatingDefinitionFieldConfigurationStyle


class RatingDefinitionFieldConfiguration(UncheckedBaseModel):
    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Color value hex code for the `style` icon symbols. ex: #fbd400
    """

    max_value: int
    style: typing.Optional[RatingDefinitionFieldConfigurationStyle] = pydantic.Field(default=None)
    """
    Field type
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .field_value_common import FieldValueCommon
from .multi_user_value_member_options_type import MultiUserValueMemberOptionsType
from .user_value import UserValue


class MultiUserValue(FieldValueCommon):
    custom_members: typing.Optional[typing.List[UserValue]] = pydantic.Field(default=None)
    """
    Populated with custom member options only if `member_options_type` is set to 'custom'.
    """

    member_options_type: typing.Optional[MultiUserValueMemberOptionsType] = None
    value: typing.Optional[typing.List[UserValue]] = None

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .share_access import ShareAccess


class Share(UncheckedBaseModel):
    access: ShareAccess
    collection_id: str = pydantic.Field()
    """
    Collection ID
    """

    commenting_enabled: bool
    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Share description
    """

    downloading_enabled: bool
    enabled: bool
    expiration: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Expiration timestamp
    """

    id: str = pydantic.Field()
    """
    Share ID
    """

    last_viewed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last viewed timestamp
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Share name
    """

    passphrase: typing.Optional[str] = pydantic.Field(default=None)
    """
    Passphrase to access share, if passphrase is required and not given it will be generated
    """

    short_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Share URL
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.unchecked_base_model import UncheckedBaseModel
from ...types.email import Email
from ...types.uuid_ import Uuid


class AddReviewersToShareParamsDataReviewers(UncheckedBaseModel):
    adobe_user_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Adobe User IDs
    """

    emails: typing.Optional[typing.List[Email]] = pydantic.Field(default=None)
    """
    Email Addresses
    """

    user_ids: typing.Optional[typing.List[Uuid]] = pydantic.Field(default=None)
    """
    User IDs
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

NumberDefinitionFieldConfigurationNumberFormat = typing.Union[
    typing.Literal["bitrate", "bits", "duration", "framerate", "frequency", "storage", "timecode"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

UpdateDateDefinitionParamsFieldConfigurationDisplayFormat = typing.Union[
    typing.Literal["local", "friendly", "usa", "euro", "iso"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel


class Workspace(UncheckedBaseModel):
    """
    Frame.io workspace
    """

    account_id: str = pydantic.Field()
    """
    Account ID
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Created Timestamp
    """

    id: str = pydantic.Field()
    """
    Workspace ID
    """

    name: str = pydantic.Field()
    """
    Workspace Name
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Updated Timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

FileWithMediaLinksInclude = typing.Union[
    typing.Literal[
        "media_links",
        "media_links.original",
        "media_links.thumbnail",
        "media_links.thumbnail_high_quality",
        "media_links.video_h264_180",
        "media_links.high_quality",
        "media_links.efficient",
        "creator",
        "project",
        "project.owner",
        "metadata",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .user import User
from .user_multi_definition import UserMultiDefinition


class UserMultiDefinitionWithIncludes(UserMultiDefinition):
    creator: typing.Optional[User] = None

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel


class Links(UncheckedBaseModel):
    """
    Links to paginated data
    """

    next: typing.Optional[str] = pydantic.Field(default=None)
    """
    "Link to next page of data.
    <br/>
    This link is the request path with the addition of the `after` query parameter to fetch the next page of results.
    Optional query parameters `page_size` and `include_total_count` are supported when paginating data."
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .account_roles_item import AccountRolesItem


class Account(UncheckedBaseModel):
    """
    Account details
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Created Timestamp
    """

    display_name: str = pydantic.Field()
    """
    Account Name
    """

    id: str = pydantic.Field()
    """
    Account ID
    """

    image: typing.Optional[str] = pydantic.Field(default=None)
    """
    The account image url
    """

    mounted_storage_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether mounted storage is enabled for this account
    """

    roles: typing.List[AccountRolesItem] = pydantic.Field()
    """
    Account User Roles
    """

    storage_limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of bytes of non-archived storage in the account. Value is nil when there is no limit
    """

    storage_usage: int = pydantic.Field()
    """
    The number of bytes of non-archived storage the account is using
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    v_4_migrated_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="v4_migrated_at"),
        pydantic.Field(alias="v4_migrated_at", default=None, description="Migration timestamp"),
    ]

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.unchecked_base_model import UncheckedBaseModel


class AddAssetParamsData(UncheckedBaseModel):
    asset_id: str = pydantic.Field()
    """
    ID for asset to be added
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.field_definition_response import FieldDefinitionResponse
from ..types.field_definitions_with_includes_response import FieldDefinitionsWithIncludesResponse
from ..types.include_total_count import IncludeTotalCount
from ..types.request_after_opaque_cursor import RequestAfterOpaqueCursor
from ..types.request_page_size import RequestPageSize
from ..types.uuid_ import Uuid
from .raw_client import AsyncRawMetadataFieldsClient, RawMetadataFieldsClient
from .types.create_field_definition_params_data import CreateFieldDefinitionParamsData
from .types.update_field_definition_params_data import UpdateFieldDefinitionParamsData

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MetadataFieldsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMetadataFieldsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMetadataFieldsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMetadataFieldsClient
        """
        return self._raw_client

    def metadata_field_definitions_delete(
        self, account_id: Uuid, field_definition_id: Uuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete account level custom field definitions. <br>Rate Limits: 60 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        field_definition_id : Uuid


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from frameio import Frameio

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata_fields.metadata_field_definitions_delete(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            field_definition_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
        )
        """
        _response = self._raw_client.metadata_field_definitions_delete(
            account_id, field_definition_id, request_options=request_options
        )
        return _response.data

    def metadata_field_definitions_update(
        self,
        account_id: Uuid,
        field_definition_id: Uuid,
        *,
        data: typing.Optional[UpdateFieldDefinitionParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionResponse:
        """
        Update account level custom field definitions. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        field_definition_id : Uuid


        data : typing.Optional[UpdateFieldDefinitionParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionResponse
            OK

        Examples
        --------
        from frameio import (
            Frameio,
            UpdateSelectDefinitionParamsFieldConfiguration,
            UpdateSelectDefinitionParamsFieldConfigurationOptionsItem,
        )
        from frameio.metadata_fields import UpdateFieldDefinitionParamsData_Select

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata_fields.metadata_field_definitions_update(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            field_definition_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            data=UpdateFieldDefinitionParamsData_Select(
                field_configuration=UpdateSelectDefinitionParamsFieldConfiguration(
                    enable_add_new=False,
                    options=[
                        UpdateSelectDefinitionParamsFieldConfigurationOptionsItem(
                            display_name="Option 1",
                        ),
                        UpdateSelectDefinitionParamsFieldConfigurationOptionsItem(
                            display_name="Option 2",
                        ),
                    ],
                ),
                name="Updated-Field-Name",
            ),
        )
        """
        _response = self._raw_client.metadata_field_definitions_update(
            account_id, field_definition_id, data=data, request_options=request_options
        )
        return _response.data

    def metadata_field_definitions_index(
        self,
        account_id: Uuid,
        *,
        include: typing.Optional[typing.Literal["creator"]] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionsWithIncludesResponse:
        """
        List account level field definitions. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        include : typing.Optional[typing.Literal["creator"]]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionsWithIncludesResponse
            OK

        Examples
        --------
        from frameio import Frameio

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata_fields.metadata_field_definitions_index(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            after="<opaque_cursor>",
            page_size=10,
            include_total_count=False,
        )
        """
        _response = self._raw_client.metadata_field_definitions_index(
            account_id,
            include=include,
            after=after,
            page_size=page_size,
            include_total_count=include_total_count,
            request_options=request_options,
        )
        return _response.data

    def metadata_field_definitions_create(
        self,
        account_id: Uuid,
        *,
        data: typing.Optional[CreateFieldDefinitionParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionResponse:
        """
        Create account level field definitions. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        data : typing.Optional[CreateFieldDefinitionParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionResponse
            Created

        Examples
        --------
        from frameio import (
            Frameio,
            SelectDefinitionParamsFieldConfiguration,
            SelectDefinitionParamsFieldConfigurationOptionsItem,
        )
        from frameio.metadata_fields import CreateFieldDefinitionParamsData_Select

        client = Frameio(
            token="YOUR_TOKEN",
        )
        client.metadata_fields.metadata_field_definitions_create(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            data=CreateFieldDefinitionParamsData_Select(
                field_configuration=SelectDefinitionParamsFieldConfiguration(
                    enable_add_new=False,
                    options=[
                        SelectDefinitionParamsFieldConfigurationOptionsItem(
                            display_name="Option 1",
                        ),
                        SelectDefinitionParamsFieldConfigurationOptionsItem(
                            display_name="Option 2",
                        ),
                    ],
                ),
                name="Fields definition name",
            ),
        )
        """
        _response = self._raw_client.metadata_field_definitions_create(
            account_id, data=data, request_options=request_options
        )
        return _response.data


class AsyncMetadataFieldsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMetadataFieldsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMetadataFieldsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMetadataFieldsClient
        """
        return self._raw_client

    async def metadata_field_definitions_delete(
        self, account_id: Uuid, field_definition_id: Uuid, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete account level custom field definitions. <br>Rate Limits: 60 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        field_definition_id : Uuid


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from frameio import AsyncFrameio

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata_fields.metadata_field_definitions_delete(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                field_definition_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.metadata_field_definitions_delete(
            account_id, field_definition_id, request_options=request_options
        )
        return _response.data

    async def metadata_field_definitions_update(
        self,
        account_id: Uuid,
        field_definition_id: Uuid,
        *,
        data: typing.Optional[UpdateFieldDefinitionParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionResponse:
        """
        Update account level custom field definitions. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        field_definition_id : Uuid


        data : typing.Optional[UpdateFieldDefinitionParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionResponse
            OK

        Examples
        --------
        import asyncio

        from frameio import (
            AsyncFrameio,
            UpdateSelectDefinitionParamsFieldConfiguration,
            UpdateSelectDefinitionParamsFieldConfigurationOptionsItem,
        )
        from frameio.metadata_fields import UpdateFieldDefinitionParamsData_Select

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata_fields.metadata_field_definitions_update(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                field_definition_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                data=UpdateFieldDefinitionParamsData_Select(
                    field_configuration=UpdateSelectDefinitionParamsFieldConfiguration(
                        enable_add_new=False,
                        options=[
                            UpdateSelectDefinitionParamsFieldConfigurationOptionsItem(
                                display_name="Option 1",
                            ),
                            UpdateSelectDefinitionParamsFieldConfigurationOptionsItem(
                                display_name="Option 2",
                            ),
                        ],
                    ),
                    name="Updated-Field-Name",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.metadata_field_definitions_update(
            account_id, field_definition_id, data=data, request_options=request_options
        )
        return _response.data

    async def metadata_field_definitions_index(
        self,
        account_id: Uuid,
        *,
        include: typing.Optional[typing.Literal["creator"]] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionsWithIncludesResponse:
        """
        List account level field definitions. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        include : typing.Optional[typing.Literal["creator"]]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionsWithIncludesResponse
            OK

        Examples
        --------
        import asyncio

        from frameio import AsyncFrameio

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata_fields.metadata_field_definitions_index(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                after="<opaque_cursor>",
                page_size=10,
                include_total_count=False,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.metadata_field_definitions_index(
            account_id,
            include=include,
            after=after,
            page_size=page_size,
            include_total_count=include_total_count,
            request_options=request_options,
        )
        return _response.data

    async def metadata_field_definitions_create(
        self,
        account_id: Uuid,
        *,
        data: typing.Optional[CreateFieldDefinitionParamsData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FieldDefinitionResponse:
        """
        Create account level field definitions. <br>Rate Limits: 10 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        data : typing.Optional[CreateFieldDefinitionParamsData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FieldDefinitionResponse
            Created

        Examples
        --------
        import asyncio

        from frameio import (
            AsyncFrameio,
            SelectDefinitionParamsFieldConfiguration,
            SelectDefinitionParamsFieldConfigurationOptionsItem,
        )
        from frameio.metadata_fields import CreateFieldDefinitionParamsData_Select

        client = AsyncFrameio(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.metadata_fields.metadata_field_definitions_create(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                data=CreateFieldDefinitionParamsData_Select(
                    field_configuration=SelectDefinitionParamsFieldConfiguration(
                        enable_add_new=False,
                        options=[
                            SelectDefinitionParamsFieldConfigurationOptionsItem(
                                display_name="Option 1",
                            ),
                            SelectDefinitionParamsFieldConfigurationOptionsItem(
                                display_name="Option 2",
                            ),
                        ],
                    ),
                    name="Fields definition name",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.metadata_field_definitions_create(
            account_id, data=data, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.unchecked_base_model import UncheckedBaseModel
from .user_value_type import UserValueType


class UserValue(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    User Id or Account User Group Id
    """

    type: UserValueType

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .comment import Comment
from .user import User


class CommentWithIncludes(Comment):
    owner: typing.Optional[User] = None
    replies: typing.Optional[typing.List[Comment]] = pydantic.Field(default=None)
    """
    Replies
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_field_definition_params_data import (
        CreateFieldDefinitionParamsData,
        CreateFieldDefinitionParamsData_Date,
        CreateFieldDefinitionParamsData_LongText,
        CreateFieldDefinitionParamsData_Number,
        CreateFieldDefinitionParamsData_Rating,
        CreateFieldDefinitionParamsData_Select,
        CreateFieldDefinitionParamsData_SelectMulti,
        CreateFieldDefinitionParamsData_Text,
        CreateFieldDefinitionParamsData_Toggle,
        CreateFieldDefinitionParamsData_UserMulti,
        CreateFieldDefinitionParamsData_UserSingle,
    )
    from .update_field_definition_params_data import (
        UpdateFieldDefinitionParamsData,
        UpdateFieldDefinitionParamsData_Date,
        UpdateFieldDefinitionParamsData_LongText,
        UpdateFieldDefinitionParamsData_Number,
        UpdateFieldDefinitionParamsData_Rating,
        UpdateFieldDefinitionParamsData_Select,
        UpdateFieldDefinitionParamsData_SelectMulti,
        UpdateFieldDefinitionParamsData_Text,
        UpdateFieldDefinitionParamsData_Toggle,
        UpdateFieldDefinitionParamsData_UserMulti,
        UpdateFieldDefinitionParamsData_UserSingle,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "CreateFieldDefinitionParamsData": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Date": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_LongText": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Number": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Rating": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Select": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_SelectMulti": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Text": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_Toggle": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_UserMulti": ".create_field_definition_params_data",
    "CreateFieldDefinitionParamsData_UserSingle": ".create_field_definition_params_data",
    "UpdateFieldDefinitionParamsData": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Date": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_LongText": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Number": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Rating": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Select": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_SelectMulti": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Text": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_Toggle": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_UserMulti": ".update_field_definition_params_data",
    "UpdateFieldDefinitionParamsData_UserSingle": ".update_field_definition_params_data",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateFieldDefinitionParamsData",
    "CreateFieldDefinitionParamsData_Date",
    "CreateFieldDefinitionParamsData_LongText",
    "CreateFieldDefinitionParamsData_Number",
    "CreateFieldDefinitionParamsData_Rating",
    "CreateFieldDefinitionParamsData_Select",
    "CreateFieldDefinitionParamsData_SelectMulti",
    "CreateFieldDefinitionParamsData_Text",
    "CreateFieldDefinitionParamsData_Toggle",
    "CreateFieldDefinitionParamsData_UserMulti",
    "CreateFieldDefinitionParamsData_UserSingle",
    "UpdateFieldDefinitionParamsData",
    "UpdateFieldDefinitionParamsData_Date",
    "UpdateFieldDefinitionParamsData_LongText",
    "UpdateFieldDefinitionParamsData_Number",
    "UpdateFieldDefinitionParamsData_Rating",
    "UpdateFieldDefinitionParamsData_Select",
    "UpdateFieldDefinitionParamsData_SelectMulti",
    "UpdateFieldDefinitionParamsData_Text",
    "UpdateFieldDefinitionParamsData_Toggle",
    "UpdateFieldDefinitionParamsData_UserMulti",
    "UpdateFieldDefinitionParamsData_UserSingle",
]
